     /**
      * \brief Flat File Parser : Configuration Interface
      *
      *
      * \author Mihael Schmidt
      * \date   07.04.2010
      *
      * \interface
      *
      */

      *---------------------------------------------------------------------------------------------
      *
      * (C) Copyleft 2010 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published by
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *---------------------------------------------------------------------------------------------


     H nomain

      /define INTERFACE


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
     D ffp_config_check...
     D                 PR
     D   configProvider...
     D                                 *   const
      *
      /copy 'config_provider_h.rpgle'
      /copy 'config_provider_t.rpgle'
      /copy 'message_h.rpgle'
      /copy 'reflection_h.rpgle'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Create configuraton provider
      *
      * Creates the configuration provider.
      *
      * \param Service program (Configuraton provider)
      * \param User data (optional)
      * \param Create procedure (default: ffp_config_create)
      *
      * \return Pointer to configuration provider
      *
      * \throws CPF9898 Incomplete implementation
      */
     P ffp_config_create...
     P                 B                   export
     D                 PI              *
     D   serviceProgram...
     D                               10A   const
     D   userdata                      *   const options(*nopass)
     D   createProcedure...
     D                              256A   const options(*nopass)
      *
     D procedurePointer...
     D                 S               *   procptr
     D create          PR              *   extproc(procedurePointer)
     D    userdata                     *   const options(*nopass)
      *
     D procedureName...
     D                 S            256A
      *
     D configProvider  S               *
     D configProviderHeader...
     D                 DS                  likeds(ffp_config_provider)
     D                                     based(configProvider)
      /free
       if (%parms() = 3);
         procedureName = createProcedure;
       else;
         procedureName = 'ffp_config_create';
       endif;

       procedurePointer = reflection_getProcedurePointer(
                                  serviceProgram :
                                  %trimr(procedureName));

       if (procedurePointer <> *null);

           // config provider aufbauen/erstellen
           if (%parms() = 1);
             configProvider = create();
           else;
             configProvider = create(userData);
           endif;

       endif;

       ffp_config_check(configProvider);

       return configProvider;
      /end-free
     P                 E


     /**
      * \brief Dispose configuration provider
      *
      * Calls clean up process and frees allocated memory.
      *
      * \param Pointer to configuration provider
      */
     P ffp_config_finalize...
     P                 B                   export
     D                 PI
     D  configProvider...
     D                                 *
      *
     D configProviderHeader...
     D                 DS                  likeds(ffp_config_provider)
     D                                     based(configProvider)
     D procedurePointer...
     D                 S               *   procptr
     D finalize        PR                  extproc(procedurePointer)
     D   configProvider...
     D                                 *
      /free
       procedurePointer = configProviderHeader.proc_finalize;

       ffp_config_check(configProvider);

       finalize(configProvider);

       if (configProvider <> *null);
         dealloc(n) configProvider;
       endif;
      /end-free
     P                 E


     /**
      * \brief Load configuration
      *
      * Loads the configuration from the configured source.
      *
      * \param Configuration provider
      * \param Parser
      */
     P ffp_config_load...
     P                 B                   export
     D                 PI
     D   configProvider...
     D                                 *   const
     D   parser                        *   const
      *
     D header          DS                  likeds(ffp_config_provider)
     D                                     based(configProvider)
      *
     D load            PR                  extproc(header.proc_load)
     D   configProvider...
     D                                 *   const
     D   parser                        *   const
      /free
       ffp_config_check(configProvider);

       load(configProvider : parser);
      /end-free
     P                 E


     /**
      * \brief Check config provider
      *
      * Checks if the config provider provides procedure pointers for all
      * procedures it should implement.
      *
      * \param Config provider
      *
      * \throws CPF9898 Incomplete implementation
      */
     P ffp_config_check...
     P                 B
     D                 PI
     D   configProvider...
     D                                 *   const
      *
     D configProviderHeader...
     D                 DS                  likeds(ffp_config_provider)
     D                                     based(configProvider)
     D cachedConfigProviderId...
     D                 S                   like(ffp_config_provider.id) static
     D configProviderChecked...
     D                 S               N   inz(*off) static
      /free
       if (configProviderHeader.id = cachedConfigProviderId and
           configProviderChecked);
         return;
       endif;

       cachedConfigProviderId = configProviderHeader.id;

       if (configProviderHeader.proc_load = *null);
         msg_sendEscapeMessage('Flat File Parser Configuration Interface: ' +
             'Incomplete Implementation ' + %trimr(configProviderHeader.id)+
             '.');
       elseif (configProviderHeader.proc_finalize = *null);
         msg_sendEscapeMessage('Flat File Parser Configuration Interface: ' +
             'Incomplete Implementation ' + %trimr(configProviderHeader.id)+
             '.');
       endif;

       configProviderChecked = *on;
      /end-free
     P                 E

