     /**
      * \brief Flat File Parser : Database Input Provider
      *
      * Die Daten werden aus der Datei MDEDATA genommen und satzweise
      * zurckgegeben.
      *
      * \author Mihael Schmidt
      * \date   24.03.2010  
      *
      * \implements FFPIN - Flat File Parser Input Provider
      */

      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------


     H nomain


      *-------------------------------------------------------------------------
      * Files
      *-------------------------------------------------------------------------
     FMDEDATA   IF   E             DISK    usropn qualified


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
      /copy 'database_input_provider_h.rpgle'


      *-------------------------------------------------------------------------
      * Data structures
      *-------------------------------------------------------------------------
      /copy 'input_provider_t.rpgle'


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
     D INPUT_PROVIDER_ID...
     D                 C                   'ffp_input_database_provider'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     P ffp_input_db_create...
     P                 B                   export
     D                 PI              *
     D   pUserData                     *   const options(*nopass)
      *
     D header          DS                  likeds(tmpl_ffp_input_provider)
     D                                     based(ptr)
      /free
       ptr = %alloc(%size(header));

       header.id = INPUT_PROVIDER_ID;
       header.userdata = *null;
       header.proc_create = %paddr('ffp_input_db_create');
       header.proc_open = %paddr('ffp_input_db_open');
       header.proc_read = %paddr('ffp_input_db_read');
       header.proc_close = %paddr('ffp_input_db_close');
       header.proc_finalize = %paddr('ffp_input_db_finalize');

       return ptr;
      /end-free
     P                 E


     P ffp_input_db_finalize...
     P                 B                   export
     D                 PI
     D   inputProvider...
     D                                 *
      *
     D header          DS                  likeds(tmpl_ffp_input_provider)
     D                                     based(inputProvider)
      /free
       ffp_input_db_close(inputProvider);

       if (header.userdata <> *null);
         dealloc header.userdata;
       endif;

       dealloc(n) inputProvider;
      /end-free
     P                 E


     P ffp_input_db_open...
     P                 B                   export
     D                 PI
     D   inputProvider...
     D                                 *   const
      /free
       if (not %open(MDEDATA));
         open MDEDATA;
       endif;

       setll *start MDEDATA;
      /end-free
     P                 E


     P ffp_input_db_close...
     P                 B                   export
     D                 PI
     D   inputProvider...
     D                                 *   const
      /free
       if (%open(MDEDATA));
         close(e) MDEDATA;
       endif;
      /end-free
     P                 E


     P ffp_input_db_read...
     P                 B                   export
     D                 PI              N
     D   inputProvider...
     D                                 *   const
     D   data                     65535A   varying
      *
     D ffp_db_in     E DS                  extname(MDEDATA : *INPUT)
     D                                     qualified
      /free
       read MDEDATA.MDEDATAF01 ffp_db_in;
       data = %trimr(ffp_db_in.data);

       return not %eof();
      /end-free
     P                 E
