     /**
      * \brief Flat File Parser : Display Logger
      *
      * All messages will be written to the job log.
      *
      * \author Mihael Schmidt
      * \date   24.12.2010
      */


      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------
      
      
     H nomain


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
      /copy 'logger_h.rpgle'
      /copy 'joblog_logger_h.rpgle'
      /copy 'message_h.rpgle'
      /copy 'libc_h.rpgle'


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
     D LOGGER_ID       C                   'ffp_logger_joblog'


      *-------------------------------------------------------------------------
      * Templates
      *-------------------------------------------------------------------------
      /copy 'logger_t.rpgle'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Create logger
      *
      * \param Userdata
      *
      * \info This logger implemenation needs no user data.
      */
     P ffp_logger_joblog_create...
     P                 B                   export
     D                 PI              *
     D   userdata                      *   const options(*nopass)
      *
     D logger          S               *
     D header          DS                  likeds(ffp_logger) based(logger)
      /free
       logger = %alloc(%size(ffp_logger));

       clear header;

       header.id = LOGGER_ID;
       header.proc_create = %paddr('ffp_logger_joblog_create');
       header.proc_log = %paddr('ffp_logger_joblog_log');
       header.proc_finalize = %paddr('ffp_logger_joblog_finalize');

       if (%parms() = 0 or userdata = *null);
         header.userdata = *null;
       else;
         header.userdata = %alloc(strlen(userdata) + 1); // speicher fr null reservieren
         memcpy(header.userdata : userdata : strlen(userdata) + 1); // null mit kopieren
       endif;

       return logger;
      /end-free
     P                 E


     /**
      * \brief Free resources
      *
      * Frees all previously allocated resources.
      *
      * \param Logger
      */
     P ffp_logger_joblog_finalize...
     P                 B                   export
     D                 PI
     D   logger                        *
      *
     D header          DS                  likeds(ffp_logger) based(logger)
      /free
       if (header.userdata <> *null);
         dealloc header.userdata;
       endif;

       dealloc(n) logger;
      /end-free
     P                 E


     /**
      * \brief Log message
      *
      * All message are written to the job log.
      *
      * \param Logger
      * \param Message
      * \param Data
      * \param Message type
      */
     P ffp_logger_joblog_log...
     P                 B                   export
     D                 PI
     D   logger                        *   const
     D   message                  65535A   const varying
     D   data                     65535A   const varying options(*omit:*nopass)
     D   type                        10I 0 const options(*nopass)
      *
     D l_message       S          65535A   varying
      /free 
       if (%parms() <= 3);
         l_message = 'INFO: ';
       elseif (type = FFP_LOG_LEVEL_INFO);
         l_message = 'INFO: ';
       elseif (type = FFP_LOG_LEVEL_FATAL);
         l_message = 'ERROR: ';
       elseif (type = FFP_LOG_LEVEL_ANNOYING);
         l_message = 'DEBUG: ';
       else;
         l_message = 'INFO: ';
       endif;
         
       l_message += %trim(message);
       
       if (%parms() >= 3 and %addr(data) <> *null);
         l_message += ' Data: ' + data;
       endif;

       msg_sendProgramMessage(l_message : MESSAGE_TYPE_INFO);
      /end-free
     P                 E
